// <copyright file="Exceptions.cs" company="Oleg V. Polikarpotchkin">
// Copyright  2009 Oleg V. Polikarpotchkin. All Right Reserved
// </copyright>
// <author>Oleg V. Polikarpotchkin</author>
// <email>ov-p@yandex.ru</email>
// <date>2009-04-12</date>
// <summary>XmlDataPointSeries Excepotions.</summary>
// <revision>$Id: Exceptions.cs 21768 2009-04-12 04:34:49Z unknown $</revision>

using System;
using System.Runtime.Serialization;
using System.Security.Permissions;

namespace OpenWPFChart.Helpers.XmlItemDataReader
{
	/// <summary>
	/// The exception is thrown when <see cref="List{Mapping}"/> contains recurring entries.
	/// </summary>
	[Serializable]
	public class RecurringMappingEntriesException : Exception, ISerializable
	{
		/// <summary>
		/// Gets the list of recurring entries.
		/// </summary>
		/// <value>The recurring entries.</value>
		public Mapping[] RecurringEntries { get; private set; }

		/// <summary>
		/// Default constructor.
		/// Initializes a new instance of the <see cref="RecurringMappingEntriesException"/> class.
		/// </summary>
		public RecurringMappingEntriesException() : base() { }
		/// <summary>
		/// Just a message constructor.
		/// Initializes a new instance of the <see cref="RecurringMappingEntriesException"/> class.
		/// </summary>
		/// <param name="message">The message string</param>
		public RecurringMappingEntriesException(String message) : base(message) { }
		/// <summary>
		/// Message with inner exception constructor.
		/// Initializes a new instance of the <see cref="RecurringMappingEntriesException"/> class.
		/// </summary>
		/// <param name="message">The message string</param>
		/// <param name="innerException">The inner exception.</param>
		public RecurringMappingEntriesException(String message, Exception innerException)
			: base(message, innerException) { }
		/// <summary>
		/// Message with Recurring Entries.
		/// Initializes a new instance of the <see cref="RecurringMappingEntriesException"/> class.
		/// </summary>
		/// <param name="message">The message string</param>
		/// <param name="recurringEntries">List of Recurring Entries.</param>
		public RecurringMappingEntriesException(String message, Mapping[] recurringEntries)
			: this(message) { RecurringEntries = recurringEntries; }
		/// <summary>
		/// Message with Recurring Entries and inner exception constructor.
		/// Initializes a new instance of the <see cref="RecurringMappingEntriesException"/> class.
		/// </summary>
		/// <param name="message">The message.</param>
		/// <param name="recurringEntries">List of Recurring Entries.</param>
		/// <param name="innerException">The inner exception.</param>
		public RecurringMappingEntriesException(String message, Mapping[] recurringEntries, Exception innerException)
			: this(message, innerException) { RecurringEntries = recurringEntries; }

		// deserialization constructor
		protected RecurringMappingEntriesException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			RecurringEntries = (Mapping[])info.GetValue("RecurringEntries", typeof(Mapping[]));
		}

		// serialization method
		[SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			// Serialize each field
			info.AddValue("RecurringEntries", RecurringEntries);

			// Let the base type serialize its fields
			base.GetObjectData(info, context);
		}
	}

	/// <summary>
	/// The exception is thrown when xml data validation failed.
	/// </summary>
	[Serializable]
	public class ValidationException : Exception, ISerializable
	{
		/// <summary>
		/// Validation Error information.
		/// </summary>
		public struct ValidationError
		{
			public string Message { get; internal set; }
			public int Line { get; internal set; }
			public int Position { get; internal set; }
		}

		/// <summary>
		/// Gets the list of Validation Errors.
		/// </summary>
		/// <value>The Validation Errors.</value>
		public ValidationError[] ValidationErrors { get; private set; }

		/// <summary>
		/// Default constructor.
		/// Initializes a new instance of the <see cref="ValidationException"/> class.
		/// </summary>
		public ValidationException() : base() { }
		/// <summary>
		/// Just a message constructor.
		/// Initializes a new instance of the <see cref="ValidationException"/> class.
		/// </summary>
		/// <param name="message">The message string</param>
		public ValidationException(String message) : base(message) { }
		/// <summary>
		/// Message with inner exception constructor.
		/// Initializes a new instance of the <see cref="ValidationException"/> class.
		/// </summary>
		/// <param name="message">The message string</param>
		/// <param name="innerException">The inner exception.</param>
		public ValidationException(String message, Exception innerException)
			: base(message, innerException) { }
		/// <summary>
		/// Message with Recurring Entries.
		/// Initializes a new instance of the <see cref="ValidationException"/> class.
		/// </summary>
		/// <param name="message">The message string</param>
		/// <param name="recurringEntries">List of Recurring Entries.</param>
		public ValidationException(String message, ValidationError[] recurringEntries)
			: this(message) { ValidationErrors = recurringEntries; }
		/// <summary>
		/// Message with Recurring Entries and inner exception constructor.
		/// Initializes a new instance of the <see cref="ValidationException"/> class.
		/// </summary>
		/// <param name="message">The message.</param>
		/// <param name="recurringEntries">List of Recurring Entries.</param>
		/// <param name="innerException">The inner exception.</param>
		public ValidationException(String message, ValidationError[] recurringEntries, Exception innerException)
			: this(message, innerException) { ValidationErrors = recurringEntries; }

		// deserialization constructor
		protected ValidationException(SerializationInfo info, StreamingContext context)
			: base(info, context)
		{
			ValidationErrors = (ValidationError[])info.GetValue("ValidationErrors", typeof(ValidationError[]));
		}

		// serialization method
		[SecurityPermission(SecurityAction.Demand, SerializationFormatter = true)]
		public override void GetObjectData(SerializationInfo info, StreamingContext context)
		{
			// Serialize each field
			info.AddValue("ValidationErrors", ValidationErrors);

			// Let the base type serialize its fields
			base.GetObjectData(info, context);
		}
	}
}