﻿// <copyright file="SampledCurveData.cs" company="Oleg V. Polikarpotchkin">
// Copyright © 2008 Oleg V. Polikarpotchkin. All Right Reserved
// </copyright>
// <author>Oleg V. Polikarpotchkin</author>
// <email>ov-p@yandex.ru</email>
// <date>2008-12-23</date>
// <summary>OpenWPFChart library. SampledCurveData class contains set of curve points.</summary>
// <revision>$Id: SampledCurveData.cs 23953 2009-05-15 06:50:11Z unknown $</revision>

using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.ComponentModel;
using System.Linq;

namespace OpenWPFChart.Parts
{
	/// <summary>
	/// <see cref="SampledCurveData{TX, TY}"/> class contains set of curve points of 
	/// <see cref="DataPoint{TX, TY}"/> type.
	/// </summary>
	/// <typeparam name="TX">The type of the abscissa.</typeparam>
	/// <typeparam name="TY">The type of the ordinate.</typeparam>
	/// <remarks>
	/// The point collection should be non-strictly ordered by abscissas either 
	/// ascending or descending.
	/// </remarks>
	public class SampledCurveData<TX, TY> : ItemData, IPoint2DSeries
	{
		#region Points
		// TODO Change type of SampledCurveData.Points property to ObservableCollection and supply appropriate TypeConverter?
		IEnumerable<DataPoint<TX, TY>> points;
		/// <summary>
		/// Gets or sets the <see cref="Points"/> property.
		/// </summary>
		/// <value><see cref="DataPoint{TAbs, TOrd}"/> iterator.</value>
		public IEnumerable<DataPoint<TX, TY>> Points
		{
			get { return points; }
			set
			{
				if (points != value)
				{
					if (value != null)
					{ // Check the value.
						if (typeof(TX).GetInterface("IComparable`1") != null)
						{ // The TX is IComparable.
							// Check whether the specified point collection is non-strictly ordered.
							if (!isNonStrictlyOrdered(value))
								throw new ArgumentException("Points must be ordered by abscissa", "value");
						}
					}

					if (points != null)
					{
						INotifyCollectionChanged iNotifyCollectionChanged = points as INotifyCollectionChanged;
						if (iNotifyCollectionChanged != null)
							iNotifyCollectionChanged.CollectionChanged -= CollectionChanged;
						else
						{
							IBindingList iBindingList = points as IBindingList;
							if (iBindingList != null)
								iBindingList.ListChanged -= ListChanged;
						}
					}

					points = value;

					if (points != null)
					{
						INotifyCollectionChanged iNotifyCollectionChanged = points as INotifyCollectionChanged;
						if (iNotifyCollectionChanged != null)
							iNotifyCollectionChanged.CollectionChanged += CollectionChanged;
						else
						{
							IBindingList iBindingList = points as IBindingList;
							if (iBindingList != null)
								iBindingList.ListChanged += ListChanged;
						}
					}

					NotifyPropertyChanged("Points");
				}
			}
		}

		/// <summary>
		/// Point observable collection changed in some way.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="System.Collections.Specialized.NotifyCollectionChangedEventArgs"/> instance containing the event data.</param>
		private void CollectionChanged(object sender, NotifyCollectionChangedEventArgs e)
		{
			NotifyPropertyChanged("Points");
		}

		/// <summary>
		/// Point binding list changed in some way.
		/// </summary>
		/// <param name="sender">The sender.</param>
		/// <param name="e">The <see cref="System.Collections.Specialized.NotifyCollectionChangedEventArgs"/> instance containing the event data.</param>
		private void ListChanged(object sender, ListChangedEventArgs e)
		{
			NotifyPropertyChanged("Points");
		}

		/// <summary>
		/// Determines whether the specified point collection is non-strictly ordered (ascending or descending).
		/// </summary>
		/// <param name="points">The points.</param>
		/// <returns>
		/// 	<c>true</c> if the specified points is ordered (non-strictly); otherwise, <c>false</c>.
		/// </returns>
		static bool isNonStrictlyOrdered(IEnumerable<DataPoint<TX, TY>> points)
		{
			bool? ordering = null;
			DataPoint<TX, TY> prevPoint = new DataPoint<TX,TY>();
			int i = 0;
			IEnumerator<DataPoint<TX, TY>> en = points.GetEnumerator();
			while (en.MoveNext())
			{
				DataPoint<TX, TY> point = en.Current;
				if (i++ > 0)
				{
					int comparison = ((IComparable<TX>)prevPoint.X).CompareTo(point.X);
					if (comparison != 0)
					{
						if (ordering.HasValue)
						{
							if (ordering.Value != comparison < 0)
								return false;
						}
						else
						{
							ordering = comparison < 0;
						}
					}
				}
				prevPoint = point;
			}
			return true;
		}
		#endregion Points

		/// <inheritdoc />
		public override Type GetDimensionType(int index)
		{
			if (index < 0 || index > 1)
				throw new ArgumentOutOfRangeException("index");
			if (index == 0)
				return typeof(TX);
			else
				return typeof(TY);
		}

		/// <inheritdoc />
		public override bool DimensionEqual(int index, ItemData item)
		{
			if (index < 0 || index > 1)
				throw new ArgumentOutOfRangeException("index");
			SampledCurveData<TX, TY> other = item as SampledCurveData<TX, TY>;
			if (other == null)
				return false;
			return (from pt in Points select pt[index]).SequenceEqual(from pt in other.Points select pt[index]);
		}

		#region IPoint2DSeries Members
		/// <inheritdoc />
		IEnumerable<Point2D> IPoint2DSeries.Points
		{
			get { return from pt in this.Points select new Point2D(pt.X, pt.Y); }
		}
		#endregion IPoint2DSeries Members

		#region ICloneable Override
		/// <summary>
		/// Creates a new object that is a copy of the current instance.
		/// </summary>
		/// <returns>
		/// A new object that is a copy of this instance.
		/// </returns>
		/// <remarks>This method is designed to provide the deep cloning facility if desired.</remarks>
		protected override object CloneInternal()
		{
			SampledCurveData<TX, TY> clone = base.CloneInternal() as SampledCurveData<TX, TY>;
			// Points.
			if (points != null)
			{
				INotifyCollectionChanged iNotifyCollectionChanged = clone.points as INotifyCollectionChanged;
				if (iNotifyCollectionChanged != null)
					iNotifyCollectionChanged.CollectionChanged += clone.CollectionChanged;
				else
				{
					IBindingList iBindingList = clone.points as IBindingList;
					if (iBindingList != null)
						iBindingList.ListChanged += clone.ListChanged;
				}
			}
			return clone;
		}
		#endregion ICloneable Override
	}
}
